<?php

class MasterBadge_ControllerAdmin_Trophy extends XFCP_MasterBadge_ControllerAdmin_Trophy
{
    // protected function _preDispatch($action)
    // {
    //  $this->_routeMatch->setSections('applications');
    // }

    public function actionIndex()
    {
        $response = parent::actionIndex();

        if ($response instanceof XenForo_ControllerResponse_View)
        {
            $viewParams =& $response->params;

            $badgeModel = $this->_getBadgeModel();

            if (empty($viewParams['trophies']))
            {
                $trophyModel = $this->_getTrophyModel();
                $viewParams['trophies'] = $trophyModel->prepareTrophies($trophyModel->getAllTrophies());
            }

            $trophies = $viewParams['trophies'] ? $viewParams['trophies'] : array();

            $badges = $badgeModel->getAllBadgesWithTrophies($trophies);
            $badges = $badgeModel->prepareBadgeTrophiesForList($badges, $totalTrophies);

            $viewParams['badges'] = $badges;
            $viewParams['totalTrophies'] = $totalTrophies;

            $response->templateName = 'MasterBadge_trophy_list';
        }

        return $response;
    }

    protected function _getTrophyAddEditResponse(array $trophy)
    {
        $response = parent::_getTrophyAddEditResponse($trophy);

        if ($response instanceof XenForo_ControllerResponse_View)
        {
            $viewParams =& $response->params;

            if (!$trophy['trophy_id'])
            {
                $viewParams['trophy']['badge_id'] = $this->_input->filterSingle('badge_id', XenForo_Input::UINT);
                $viewParams['trophy']['trophy_order'] = 10;
            }

            $viewParams['badgeOptions'] = $this->_getBadgeModel()->getBadgeOptions();

            $badgeModel = $this->_getBadgeModel();
            $trophyModel = $this->_getTrophyModel();
            
            $viewParams['trophies'] = $trophyModel->prepareTrophies($trophyModel->getAllTrophies());

            $badges = $badgeModel->getAllBadgesWithTrophies($viewParams['trophies']);
            $badges = $badgeModel->prepareBadgeTrophiesForList($badges, $totalTrophies);

            $viewParams['badges'] = $badges;
            $viewParams['totalTrophies'] = $totalTrophies;
        }

        return $response;
    }

    public function actionSave()
    {
        $GLOBALS['MasterBadge_ControllerAdmin_Trophy'] = $this;
        
        $response = parent::actionSave();

        if ($response instanceof XenForo_ControllerResponse_Redirect)
        {
            $session = XenForo_Application::getSession();
            $uploadTrophyIcon = XenForo_Application::getSession()->get('uploadTrophyIcon');
            if ($uploadTrophyIcon)
            {
                $session->remove('uploadTrophyIcon');
                return $this->responseRedirect(
                    XenForo_ControllerResponse_Redirect::SUCCESS,
                    XenForo_Link::buildAdminLink('trophies/icon', $uploadTrophyIcon)
                );
            }
        }

        return $response;
    }

    public function MasterBadge_actionSave(XenForo_DataWriter_Trophy $dw)
    {       
        $input = $this->_input->filter(array(
            'trophy_key' => XenForo_Input::STRING,
            'badge_id' => XenForo_Input::UINT,
            'trophy_order' => XenForo_Input::UINT,
            'callback_class' => XenForo_Input::STRING,
            'callback_method' => XenForo_Input::STRING,
        ));

        if ($this->_input->filterSingle('upload_trophy_icon', XenForo_Input::BOOLEAN))
        {
            $dw->setOption(MasterBadge_DataWriter_Trophy::OPTION_UPLOAD_ICON, true);
        }

        if ($input['callback_class'] || $input['callback_method'])
        {
            $class = $input['callback_class'];
            $method = $input['callback_method'];

            if (!XenForo_Helper_Php::validateCallbackPhrased($class, $method, $errorPhrase))
            {
                $dw->error($errorPhrase, 'callback_method');
            }
        }

        $dw->bulkSet($input);

        $icon = XenForo_Upload::getUploadedFile('trophy_icon');
        if (!empty($icon)) 
        {
            $dw->setImage($icon);
        }
    }

    public function actionIcon()
    {
        $trophyId = $this->_input->filterSingle('trophy_id', XenForo_Input::UINT);
        $trophy = $this->_getTrophyOrError($trophyId);

        if ($this->isConfirmedPost())
        {
            if ($this->_input->filterSingle('delete_trophy_icon', XenForo_Input::BOOLEAN))
            {
                $this->_getTrophyModel()->deleteTrophyIcon($trophy);
            }
            else
            {
                $dw = XenForo_DataWriter::create('XenForo_DataWriter_Trophy');
                $dw->setExistingData($trophy);

                $icon = XenForo_Upload::getUploadedFile('trophy_icon');
                if (!empty($icon)) 
                {
                    $dw->setImage($icon);
                }

                $dw->save();
            }

            return $this->responseRedirect(
                XenForo_ControllerResponse_Redirect::SUCCESS,
                XenForo_Link::buildAdminLink('trophies') . $this->getLastHash($trophyId)
            );
        }
        else
        {
            $viewParams = array(
                'trophy' => $trophy
            );

            return $this->responseView('MasterBadge_ViewAdmin_Trophy_Icon', 'MasterBadge_trophy_icon', $viewParams);
        }
    }

    public function actionAward()
    {
        $trophyId = $this->_input->filterSingle('trophy_id', XenForo_Input::UINT);
        $trophy = $this->_getTrophyOrError($trophyId);

        if ($this->isConfirmedPost())
        {
            $username = $this->_input->filterSingle('username', XenForo_Input::STRING);

            if ($username !== '')
            {
                $users = $this->getModelFromCache('XenForo_Model_User')->getUsersByNames(explode(',', $username), array(
                    'join' => XenForo_Model_User::FETCH_USER_FULL
                ));

                if ($users)
                {

                    foreach ($users as $user) 
                    {
                        MasterBadge_Core::awardUserTrophy($user, $trophy);
                    }
                }
            }

            return $this->responseRedirect(
                XenForo_ControllerResponse_Redirect::SUCCESS,
                XenForo_Link::buildAdminLink('trophies') . $this->getLastHash($trophyId)
            );
        }
        else
        {
            $viewParams = array(
                'trophy' => $trophy
            );

            return $this->responseView('MasterBadge_ViewAdmin_Trophy_Award', 'MasterBadge_trophy_award', $viewParams);
        }
    }

    public function actionRemove()
    {
        $trophyId = $this->_input->filterSingle('trophy_id', XenForo_Input::UINT);
        $userId = $this->_input->filterSingle('user_id', XenForo_Input::UINT);
        $trophy = $this->_getTrophyOrError($trophyId);

        if (!$userId)
        {
            return $this->responseError(new XenForo_Phrase('requested_user_not_found'), 404);
        }

        if ($this->isConfirmedPost())
        {
            $this->_getTrophyModel()->removeUserTrophy($userId, $trophyId);

            return $this->responseRedirect(
                XenForo_ControllerResponse_Redirect::SUCCESS,
                XenForo_Link::buildAdminLink('trophies/users', $trophy)
            );
        }
        else
        {
            $user = $this->getModelFromCache('XenForo_Model_User')->getUserById($userId);

            if (!$user)
            {
                return $this->responseError(new XenForo_Phrase('requested_user_not_found'), 404);
            }

            $viewParams = array(
                'trophy' => $trophy,
                'user' => $user
            );

            return $this->responseView('MasterBadge_ViewAdmin_Trophy_Remove', 'MasterBadge_trophy_remove', $viewParams);
        }
    }

    public function actionUsers()
    {
        $trophyId = $this->_input->filterSingle('trophy_id', XenForo_Input::UINT);
        $trophy = $this->_getTrophyOrError($trophyId);

        $page = $this->_input->filterSingle('page', XenForo_Input::UINT);
        $perPage = 20;
        $pageNavParams = array();

        $fetchOptions = array(
            'page' => $page,
            'perPage' => $perPage
        );

        $orderBy = $this->_input->filterSingle('order', XenForo_Input::STRING);
        $orderDirection = $this->_input->filterSingle('direction', XenForo_Input::STRING);
        if ($orderBy)
        {
            $fetchOptions['order'] = $orderBy;
            $fetchOptions['direction'] = $orderDirection;
            $pageNavParams['order'] = $orderBy;
            $pageNavParams['direction'] = $orderDirection;
        }

        $trophyModel = $this->_getTrophyModel();
        
        $username = $this->_input->filterSingle('username', XenForo_Input::STRING);

        if ($username)
        {
            $user = $this->_getUserModel()->getUserByName($username);
        }
        else
        {
            $user = null;
        }

        $conditions = array(
            'trophy_id' => $trophyId
        );
        if ($user)
        {
            $conditions['user_id'] = $user['user_id'];
            $pageNavParams['username'] = $user['username'];
        }

        $total = $trophyModel->countTrophyUsers($conditions);

        $viewParams = array(
            'trophy' => $trophy,
            'users' => $trophyModel->getUsersByTrophyId($trophyId, $fetchOptions),

            'total' => $total,
            'perPage' => $perPage,
            'page' => $page,
            'pageNavParams' => $pageNavParams
        );

        return $this->responseView('MasterBadge_ViewAdmin_Trophy_Users', 'MasterBadge_trophy_users', $viewParams);
    }

    public function actionTrophyOrder()
    {
        $order = $this->_input->filterSingle('order', XenForo_Input::ARRAY_SIMPLE);

        $this->_assertPostOnly();
        $this->_getTrophyModel()->massUpdateTrophyOrder($order);

        return $this->responseRedirect(
            XenForo_ControllerResponse_Redirect::SUCCESS,
            XenForo_Link::buildAdminLink('trophies')
        );
    }

    public function actionImport()
    {
        if ($this->isConfirmedPost())
        {
            if ($_input = $this->_getInputFromSerialized('_xfTrophyImportData', true))
            {
                $this->_input = $_input;
            }

            $input = $this->_input->filter(array(
                'badges' => XenForo_Input::ARRAY_SIMPLE,
                'import' => XenForo_Input::ARRAY_SIMPLE,
                'trophies' => XenForo_Input::ARRAY_SIMPLE,
            ));

            $trophies = array();

            foreach ($input['import'] AS $trophyId)
            {
                if (empty($input['trophies'][$trophyId]))
                {
                    continue;
                }

                $trophyInput = new XenForo_Input($input['trophies'][$trophyId]);

                $trophies[$trophyId] = $trophyInput->filter(array(
                    'title' => XenForo_Input::STRING,
                    'description' => XenForo_Input::STRING,
                    'trophy_points' => XenForo_Input::UINT,
                    'trophy_key' => XenForo_Input::STRING,
                    'user_criteria' => XenForo_Input::ARRAY_SIMPLE,
                    'badge_id' => XenForo_Input::STRING,
                    'trophy_order' => XenForo_Input::UINT,
                    'callback_class' => XenForo_Input::STRING,
                    'callback_method' => XenForo_Input::STRING,
                ));
            }

            $this->_getTrophyModel()->massImportTrophies($trophies, $input['badges'], $errors);

            if (empty($errors))
            {
                return $this->responseRedirect(
                    XenForo_ControllerResponse_Redirect::SUCCESS,
                    XenForo_Link::buildAdminLink('trophies')
                );
            }
            else
            {
                return $this->responseError($errors);
            }
        }
        else
        {
            return $this->responseView('MasterBadge_ViewAdmin_Trophy_Import', 'MasterBadge_trophy_import', array());
        }
    }

    public function actionValidateImportField()
    {
        $field = $this->_getFieldValidationInputParams();

        $fieldName = preg_replace('/^(.+)__\d+$/si', '\1', $field['name']);

        $dw = XenForo_DataWriter::create('XenForo_DataWriter_Trophy');
        $dw->set($fieldName, $field['value']);

        if ($errors = $dw->getErrors())
        {
            $errors = array($field['name'] => $errors[$fieldName]);

            return $this->responseError($errors);
        }

        return $this->responseRedirect(
            XenForo_ControllerResponse_Redirect::SUCCESS,
            '',
            new XenForo_Phrase('redirect_field_validated', array('name' => $field['name'], 'value' => $field['value']))
        );
    }

    public function actionImportForm()
    {
        $this->_assertPostOnly();

        $input = $this->_input->filter(array(
            'mode' => XenForo_Input::STRING,
            'server_file' => XenForo_Input::STRING,
        ));

        $badgeOptions = $this->_getBadgeModel()->getBadgeOptions();

        if ($input['mode'] == 'upload')
        {
            $upload = XenForo_Upload::getUploadedFile('upload');
            if (!$upload)
            {
                return $this->responseError(new XenForo_Phrase('please_upload_valid_smilies_xml_file'));
            }
        }
        else
        {
            $upload = $input['server_file'];
        }

        $document = $this->getHelper('Xml')->getXmlFromFile($upload);
        $trophyData = $this->_getTrophyModel()->getTrophyDataFromXml($document, $badgeOptions);

        $viewParams = array(
            'rules' => $trophyData['rules'],
            'trophies' => $trophyData['trophies'],
            'newBadges' => $trophyData['newBadges'],
            'newBadgeOptions' => $trophyData['newBadgeOptions'],
            'badgeOptions' => $badgeOptions,

            'userCriteriaData' => XenForo_Helper_Criteria::getDataForUserCriteriaSelection(),
        );

        return $this->responseView('MasterBadge_ViewAdmin_Trophy_ImportForm', 'MasterBadge_trophy_import_form', $viewParams);
    }

    public function actionExport()
    {
        $this->_assertPostOnly();

        $trophyIds = $this->_input->filterSingle('trophyId', XenForo_Input::ARRAY_SIMPLE);

        if (!$trophyIds)
        {
            return $this->responseError(new XenForo_Phrase('MasterBadge_please_select_at_least_one_trophy'));
        }

        $this->_routeMatch->setResponseType('xml');

        $viewParams = array(
            'xml' => $this->_getTrophyModel()->getTrophiesXml($trophyIds)
        );

        return $this->responseView('MasterBadge_ViewAdmin_Trophy_Export', '', $viewParams);
    }

    protected function _getBadgeModel()
    {
        return $this->getModelFromCache('MasterBadge_Model_Badge');
    }

    protected function _getUserModel()
    {
        return $this->getModelFromCache('XenForo_Model_User');
    }
}