<?php

class MasterBadge_ControllerPublic_Badge extends XenForo_ControllerPublic_Abstract
{
    public function actionIndex()
    {
        $userId = $this->_input->filterSingle('user_id', XenForo_Input::UINT);
        $badgeId = $this->_input->filterSingle('badge_id', XenForo_Input::UINT);

        if ($userId && $badgeId)
        {
            return $this->responseReroute(__CLASS__, 'view');
        }

        $userModel = $this->_getUserModel();
        $trophyModel = $this->_getTrophyModel();
        $badgeModel = $this->_getBadgeModel();
        $limit = XenForo_Application::get('options')->membersPerPage;
        $day = XenForo_Locale::getDayStartTimestamps();

        $users = $userModel->getUsers(array(
            'is_banned' => 0,
            'trophy_points' => array('>', 0)
        ), array(
            'join' => XenForo_Model_User::FETCH_USER_FULL,
            'limit' => $limit,
            'order' => 'trophy_points',
            'direction' => 'desc'
        ));

        $latestAwarded = $trophyModel->getLatestAwarded(array(), array('limit' => $limit));

        $badges = $badgeModel->getBadgeWithTrophiesForCache();
        $badges = $badgeModel->prepareBadgeForUser($badges, XenForo_Visitor::getInstance()->toArray());     

        $badgeUsersCount = $trophyModel->countBadgeUsers(array(
            'badge_id' => array_keys($badges)
        ));

        $badgeAwardedCount = $trophyModel->getLatestAwardedCount($day['week'], $day['now']);

        foreach ($badges as $badgeId => &$badge) 
        {
            if (!$badgeId || in_array($badge['type'], array('step', 'hidden')))
            {
                unset($badges[$badgeId]);
            }
            else
            {
                $badge['totalUsers'] = isset($badgeUsersCount[$badgeId]['total'])
                    ? $badgeUsersCount[$badgeId]['total']
                    : 0;

                $badge['awardedCount'] = isset($badgeAwardedCount[$badgeId]['total'])
                    ? $badgeAwardedCount[$badgeId]['total']
                    : 0;

                $_limit = ($badge['totalUsers'] < 4) ? $badge['totalUsers'] : 3;

                $badgeUsers = $trophyModel->getLatestAwarded(array(
                    'badge_id' => $badgeId
                ), array(
                    'limit' => $_limit,
                    'extra' => 'GROUP BY user_trophy.user_id'
                ));

                $badge['users'] = array();

                foreach ($badgeUsers as $badgeUser) 
                {
                    $badge['users'][$badgeUser['user_id']] = $badgeUser;
                }               

                $badge['moreUsers'] = ($badge['totalUsers'] - count($badge['users'])) > 0
                    ? $badge['totalUsers'] - count($badge['users'])
                    : 0;
            }
        }

        $viewParams = array(
            'badges' => $badges,
            'users' => $users,
            'latestAwarded' => $trophyModel->prepareTrophies($latestAwarded),
        ) + $this->_getDefaultViewParams();

        return $this->responseView('MasterBadge_ViewPublic_Badge_Index', 'MasterBadge_badge_index', $viewParams);
    }

    public function actionStats()
    {
        return $this->responseMessage('Coming Soon');

        $badgeModel = $this->_getBadgeModel();
        $trophyModel = $this->_getTrophyModel();

        $typeStats = $badgeModel->getBadgeTypeStats();
        $_typeStats = array();

        foreach ($typeStats as $stats) 
        {
            $_typeStats[$stats['type']] = array((int) $stats['total']);
        }

        $badges = $badgeModel->getBadges();
        $badgeTypes = array();

        $time = XenForo_Application::$time;

        $latestAwarded = $trophyModel->getLatestAwarded();
        $latestAwardedStats = array(
            'visible' => 0,
            'step' => 0,
            'hidden' => 0
        );

// $trophyModel->getLatestAwardedData(strtotime('-1 year'), XenForo_Application::$time);

        foreach ($latestAwarded as $awarded) 
        {
            if ($awarded['badge_id'])
            {
                $latestAwardedStats[$badges[$awarded['badge_id']]['type']]++;
            }
            else
            {
                $latestAwardedStats['hidden']++;
            }
        }

        $viewParams = array(
            'typeStats' => $_typeStats,
            'latestAwardedStats' => $latestAwardedStats
        ) + $this->_getDefaultViewParams();

        return $this->responseView('MasterBadge_ViewPublic_Badge_Stats', 'MasterBadge_badge_stats', $viewParams);
    }

    public function _getDefaultViewParams()
    {
        $visitor = XenForo_Visitor::getInstance()->toArray();
        $userModel = $this->_getUserModel();
        
        if ($visitor['user_id'] && isset($visitor['following']))
        {
            $followingToShowCount = MasterBadge_Const::FOLLOWING_TO_SHOW_COUNT;
            $followingCount = substr_count($visitor['following'], ',') + 1;

            $following = $userModel->getFollowedUserProfiles($visitor['user_id'], $followingToShowCount, 'user.trophy_points DESC');

            if (($followingCount >= $followingToShowCount && count($following) < $followingToShowCount)
                || ($followingCount < $followingToShowCount && $followingCount != count($following)))
            {
                // following count is off, rebuild it
                $visitor['following'] = $userModel->getFollowingDenormalizedValue($visitor['user_id']);
                $userModel->updateFollowingDenormalizedValue($visitor['user_id'], $visitor['following']);

                $followingCount = substr_count($visitor['following'], ',') + 1;
            }
        }
        else
        {
            $followingCount = 0;

            $following = array();
        }   

        if ($following) 
        {
            foreach ($following as &$followUser) 
            {                
                $followUser['canViewActivity'] = method_exists($userModel, 'canViewUserCurrentActivity') ? $userModel->canViewUserCurrentActivity($followUser) : false;
            }
        }

        return array(
            'followingCount' => $followingCount,
            'following' => $following,
        );
    }

    public function actionGraph()
    {
        $badgeId = $this->_input->filterSingle('badge_id', XenForo_Input::UINT);
        $badge = $this->_getBadgeOrError($badgeId);

        if (!$this->_noRedirect())
        {
            return $this->responseRedirect(
                XenForo_ControllerResponse_Redirect::RESOURCE_CANONICAL,
                XenForo_Link::buildPublicLink('members/badges', XenForo_Visitor::getInstance()->toArray(), 
                    array('badge_id' => $badgeId))
            );
        }

        $trophyModel = $this->_getTrophyModel();
        $day = XenForo_Locale::getDayStartTimestamps();

        // $graphData = $trophyModel->getLatestAwardedGraphData($badgeId, $day['week'] - (86400 * 90), $day['now']);
        $graphData = $trophyModel->getLatestAwarded(array(
            'badge_id' => $badgeId,
            'award_date' => array('>=<', $day['week'], $day['now'])
        ));

        $graphData = $trophyModel->filterGraphData($graphData, $day['week'], $day['now']);

        $viewParams = array(
            'badgeId' => $badgeId,

            'graphData' => $trophyModel->prepareGraphData($graphData)
        );

        return $this->responseView('MasterBadge_ViewPublic_Badge_GraphData', 'MasterBadge_badge_graph_data', $viewParams); 
    }

    public function actionView()
    {
        $userId = $this->_input->filterSingle('user_id', XenForo_Input::UINT);
        $user = $this->getHelper('UserProfile')->assertUserProfileValidAndViewable($userId);
        
        $badgeId = $this->_input->filterSingle('badge_id', XenForo_Input::UINT);
        $badge = $this->_getBadgeOrError($badgeId);

        $this->canonicalizeRequestUrl(XenForo_Link::buildPublicLink('members/badges', $user, $badge));

        $badgeModel = $this->_getBadgeModel();

        // TODO: Fix this permission
        /*if (!$badgeModel->canViewBadge($badge, $error))
        {
            throw $this->getErrorOrNoPermissionResponseException($error);
        }*/

        $badges = $badgeModel->getBadgeWithTrophiesForCache();
        $badges = $badgeModel->prepareBadgeForUser($badges, $user);

        $viewParams = array(
            'user' => $user,
            'badge' => $badges[$badgeId]
        );

        return $this->responseView('MasterBadge_ViewPublic_Badge_View', 'MasterBadge_badge_view', $viewParams); 
    }

    public static function getSessionActivityDetailsForList(array $activities)
    {
        if (!XenForo_Visitor::getInstance()->hasPermission('general', 'viewProfile'))
        {
            return new XenForo_Phrase('viewing_members');
        }

        $userIds = array();
        foreach ($activities AS $activity)
        {           
            if (!empty($activity['params']['user_id']))
            {
                $userIds[$activity['params']['user_id']] = intval($activity['params']['user_id']);
            }

            if (!empty($activity['params']['badge_id']))
            {
                $badgeId = intval($activity['params']['badge_id']);
            }
        }

        $userData = array();

        if ($userIds)
        {
            /* @var $userModel XenForo_Model_User */
            $userModel = XenForo_Model::create('XenForo_Model_User');

            $users = $userModel->getUsersByIds($userIds, array(
                'join' => XenForo_Model_User::FETCH_USER_PRIVACY
            ));
            foreach ($users AS $user)
            {
                $userData[$user['user_id']] = array(
                    'username' => $user['username'],
                    'url' => (!empty($badgeId)) 
                        ? XenForo_Link::buildPublicLink('members/badges', $user, array('badge_id' => $badgeId))
                        : XenForo_Link::buildPublicLink('members/badges', $user),
                );
            }
        }

        $output = array();
        foreach ($activities AS $key => $activity)
        {
            $user = false;
            if (!empty($activity['params']['user_id']))
            {
                $userId = $activity['params']['user_id'];
                if (isset($userData[$userId]))
                {
                    $user = $userData[$userId];
                }
            }

            if ($user)
            {
                $output[$key] = array(
                    new XenForo_Phrase('MasterBadge_viewing_x_badge', array('user' => $user['username'])),
                    utf8_strtolower(new XenForo_Phrase('MasterBadge_badge')),
                    $user['url'],
                    false
                );
            }
            else
            {
                $output[$key] = new XenForo_Phrase('MasterBadge_badges');
            }
        }

        return $output;
    }

    protected function _getBadgeOrError($id = null, array $fetchOptions = array())
    {
        if ($id === null)
        {
            $id = $this->_input->filterSingle('badge_id', XenForo_Input::UINT);
        }

        $badge = $this->_getBadgeModel()->getBadgeById($id, $fetchOptions);

        if (!$badge)
        {
            throw $this->responseException($this->responseError(new XenForo_Phrase('MasterBadge_badge_not_found'), 404));
        }

        return $badge;
    }

    protected function _getBadgeModel()
    {
        return $this->getModelFromCache('MasterBadge_Model_Badge');
    }

    protected function _getTrophyModel()
    {
        return $this->getModelFromCache('XenForo_Model_Trophy');
    }

    protected function _getUserModel()
    {
        return $this->getModelFromCache('XenForo_Model_User');
    }
}