<?php

class MasterBadge_ControllerPublic_Trophy extends XenForo_ControllerPublic_Abstract
{
    public function actionIndex()
    {
        $userId = $this->_input->filterSingle('user_id', XenForo_Input::UINT);
        $trophyId = $this->_input->filterSingle('trophy_id', XenForo_Input::UINT);

        if ($userId && $trophyId)
        {
            return $this->responseReroute(__CLASS__, 'view');
        }

        $visitor = XenForo_Visitor::getInstance()->toArray();
        $redirect = !empty($visitor['user_id']) 
            ? XenForo_Link::buildPublicLink('members/badges', $visitor)
            : XenForo_Link::buildPublicLink('members');

        return $this->responseRedirect(
            XenForo_ControllerResponse_Redirect::SUCCESS,
            $redirect
        );
    }

    public function actionView()
    {
        $userId = $this->_input->filterSingle('user_id', XenForo_Input::UINT);
        $user = $this->getHelper('UserProfile')->assertUserProfileValidAndViewable($userId);

        $trophyId = $this->_input->filterSingle('trophy_id', XenForo_Input::UINT);
        $trophy = $this->_getTrophyOrError($trophyId);

        // if ($this->_noRedirect())
        // {
        //  return $this->responseReroute('XenForo_ControllerPublic_Member', 'trophies');
        // }

        if (!empty($trophy['badge_id']))
        {
            return $this->responseRedirect(
                XenForo_ControllerResponse_Redirect::SUCCESS,
                XenForo_Link::buildPublicLink('members/badges', $user, array('badge_id' => $trophy['badge_id'])) . '#trophy-' . $trophyId
            );
        }

        $trophyModel = $this->_getTrophyModel();

        $this->canonicalizeRequestUrl(XenForo_Link::buildPublicLink('members/trophies', $user, $trophy));
        //$awarded = $trophyModel->getTrophiesForUserId(XenForo_Visitor::getUserId());

        $trophy = $trophyModel->prepareTrophyForUser($trophy, $user);

        $viewParams = array(
            'user' => $user,
            'trophy' => $trophyModel->prepareTrophy($trophy)
        );

        return $this->responseView('MasterBadge_ViewPublic_Member_Trophy', 'MasterBadge_trophy_view', $viewParams);
    }

    public function actionFeatured()
    {
        if (!XenForo_Application::getOptions()->MasterBadge_featured)
        {
            return $this->responseNoPermission();
        }

        $this->_assertRegistrationRequired();

        $trophyId = $this->_input->filterSingle('trophy_id', XenForo_Input::UINT);

        $visitor = XenForo_Visitor::getInstance();

        $dw = XenForo_DataWriter::create('XenForo_DataWriter_User');
        $dw->setExistingData($visitor['user_id']);

        if ($this->_request->isPost())
        {
            $order = $this->_input->filterSingle('order', XenForo_Input::ARRAY_SIMPLE);

            $trophyModel = $this->_getTrophyModel();

            foreach($order as $key => $trophyId)
            {
                $trophy = $trophyModel->getTrophyFromCache($trophyId);
                $trophy = $trophyModel->prepareTrophyForUser($trophy);

                if (empty($trophy['award_date']))
                {
                    unset($order[$key]);
                }
            }
            $dw->set('favorite_badge', $order);
        }
        else
        {
            if (!$trophyId)
            {
                $this->_checkCsrfFromToken($this->_input->filterSingle('t', XenForo_Input::STRING));
                $dw->set('favorite_badge', false);                
            }
        }

        $dw->save();

        return $this->responseRedirect(
            XenForo_ControllerResponse_Redirect::SUCCESS,
            $this->getDynamicRedirect()
        );
    }

    public static function getSessionActivityDetailsForList(array $activities)
    {
        if (!XenForo_Visitor::getInstance()->hasPermission('general', 'viewProfile'))
        {
            return new XenForo_Phrase('viewing_members');
        }

        $userIds = array();
        foreach ($activities AS $activity)
        {           
            if (!empty($activity['params']['user_id']))
            {
                $userIds[$activity['params']['user_id']] = intval($activity['params']['user_id']);
            }

            if (!empty($activity['params']['trophy_id']))
            {
                $trophyId = intval($activity['params']['trophy_id']);
            }
        }

        $userData = array();

        if ($userIds)
        {
            /* @var $userModel XenForo_Model_User */
            $userModel = XenForo_Model::create('XenForo_Model_User');

            $users = $userModel->getUsersByIds($userIds, array(
                'join' => XenForo_Model_User::FETCH_USER_PRIVACY
            ));
            foreach ($users AS $user)
            {
                $userData[$user['user_id']] = array(
                    'username' => $user['username'],
                    'url' => (!empty($trophyId)) 
                        ? XenForo_Link::buildPublicLink('members/trophies', $user, array('trophy_id' => $trophyId))
                        : XenForo_Link::buildPublicLink('members/badges', $user),
                );
            }
        }

        $output = array();
        foreach ($activities AS $key => $activity)
        {
            $user = false;
            if (!empty($activity['params']['user_id']))
            {
                $userId = $activity['params']['user_id'];
                if (isset($userData[$userId]))
                {
                    $user = $userData[$userId];
                }
            }

            if ($user)
            {
                $output[$key] = array(
                    new XenForo_Phrase('MasterBadge_viewing_x_badge', array('user' => $user['username'])),
                    utf8_strtolower(new XenForo_Phrase('MasterBadge_badge')),
                    $user['url'],
                    false
                );
            }
            else
            {
                $output[$key] = new XenForo_Phrase('MasterBadge_badges');
            }
        }

        return $output;
    }

    protected function _getTrophyOrError($id = null, array $fetchOptions = array())
    {
        if ($id === null)
        {
            $id = $this->_input->filterSingle('trophy_id', XenForo_Input::UINT);
        }

        $trophy = $this->_getTrophyModel()->getTrophyById($id, $fetchOptions);

        if (!$trophy)
        {
            throw $this->responseException($this->responseError(new XenForo_Phrase('requested_trophy_not_found'), 404));
        }

        return $trophy;
    }

    protected function _getTrophyModel()
    {
        return $this->getModelFromCache('XenForo_Model_Trophy');
    }
}