<?php

class MasterBadge_Core
{
    public static function awardUserTrophy($user, $trophy, $awardDate = null)
    {
        if (!is_array($user))
        {
            $user = XenForo_Model::create('XenForo_Model_User')->getUserById($user);
            if (!$user)
            {
                return false;
            }
        }

        $trophyModel = XenForo_Model::create('XenForo_Model_Trophy');
        $userTrophies = $trophyModel->getTrophiesForUserId($user['user_id']);

        if (is_array($trophy))
        {
            if (!isset($userTrophies[$trophy['trophy_id']]))
            {
                $trophyModel->awardUserTrophy($user, $user['username'], $trophy, $awardDate);

                return true;
            }
        }
        else
        {
            $_trophy = $trophyModel->getTrophyByTrophyKey($trophy);

            if ($_trophy)
            {
                if (!isset($userTrophies[$_trophy['trophy_id']]))
                {
                    $trophyModel->awardUserTrophy($user, $user['username'], $_trophy, $awardDate);

                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Allias of awardUserTrophy
     */
    public static function award($user, $trophy, $awardDate = null)
    {
        return static::awardUserTrophy($user, $trophy, $awardDate);
    }

    /**
     * Allias of awardUserTrophy 
     */
    public static function unlock($user, $trophy, $awardDate = null)
    {
        return static::awardUserTrophy($user, $trophy, $awardDate);
    }

    public static function getLevel($points)
    {
        if (isset(XenForo_Template_Helper_Core::$helperCallbacks['level']))
        {
            return XenForo_Template_Helper_Core::callHelper('level', array($points));
        }

        return MasterBadge_Helper_Level::getLevelByPoints($points);
    }

    public static function getUserLevel(array $user, $points = null)
    {
        if ($points !== null)
        {
            return static::getLevel($points);
        }

        return static::getLevel($user['trophy_points']);
    }

    public static function getLevelHtml($points)
    {
        $level = static::getLevel($points);

        $numbers = str_split($level, 1);
        $count = count($numbers) - 1;

        $levelClass = '';

        foreach ($numbers as $key => $value) 
        {
            $x = $value * pow(10, $count);
            $plus = str_repeat('plus_', $key);
            $count--;
            if ($count >= 0)
            {
                $levelClass .= 'lvl_' . $plus . $x . ' ';
            }
        }

        $levelClass .= 'lvl' . $level;
        
        $class = htmlspecialchars(rtrim($levelClass));
        return "<div class=\"userLevel {$class}\"><span class=\"userLevelNum\">{$level}</span></div>";
    }

    public static function getUserLevelHtml(array $user, $points = null)
    {
        if ($points !== null)
        {
            return static::getLevelHtml($points);
        }
        
        return static::getLevelHtml($user['trophy_points']);
    }   

    public static function getUserLevelProgress(array $user)
    {
        $progress = array();

        if ($user['user_id'])
        {
            $progress['currentLevel'] = $currentLevel = MasterBadge_Core::getUserLevel($user);
            $progress['currentLevelPoint'] = $currentLevelPoint = MasterBadge_Helper_Level::getPointsByLevel($currentLevel);            

            if ($maxLevel = XenForo_Application::getOptions()->MasterBadge_maxLevel)
            {                
                if ($currentLevel >= $maxLevel)
                {
                    return array_merge($progress, array(
                        'nextLevel' => $maxLevel,
                        'nextLevelPoint' => MasterBadge_Helper_Level::getPointsByLevel($maxLevel),
                        'percentage' => 100,
                        'maximum' => true
                    ));
                }
            }

            $nextLevel = $currentLevel + 1;
            $points = MasterBadge_Helper_Level::getPointsByLevel($nextLevel) - MasterBadge_Helper_Level::getPointsByLevel($currentLevel);
            $earnedPoints = $user['trophy_points'] - MasterBadge_Helper_Level::getPointsByLevel(MasterBadge_Core::getUserLevel($user));
            $earnedPoints = ($earnedPoints > 0) ? $earnedPoints : 0;

            return array_merge($progress, array(
                'nextLevel' => $nextLevel,
                'nextLevelPoint' => MasterBadge_Helper_Level::getPointsByLevel($nextLevel),
                'percentage' => ($earnedPoints / $points) * 100,
                'maximum' => false
            ));                
        }

        return $progress;
    }

    public static function helperUserTitle($user, $allowCustomTitle = true, $withBanner = false)
    {
        if (isset($user['trophy_points']))
        {
            $user['level'] = MasterBadge_Helper_Level::getLevelByPoints($user['trophy_points']);
        }

        return XenForo_Template_Helper_Core::helperUserTitle($user, $allowCustomTitle, $withBanner);
    }

    public static function getFeaturedBadges($level)
    {
        $featuredBadges = XenForo_Application::getOptions()->MasterBadge_featuredBadges;
        $badges = 0;
        $i = 0;

        while ($i < count($featuredBadges)) 
        {
            $nextLevel = isset($featuredBadges[$i + 1]['level']) ? $featuredBadges[$i + 1]['level'] : $featuredBadges[$i]['level'];
            if ($featuredBadges[$i]['level'] <= $level && $nextLevel > $level) 
            {
                $badges = $featuredBadges[$i]['badge'];
            } 
            else if ($featuredBadges[$i]['level'] <= $level) 
            {
                $badges = isset($featuredBadges[$i]['badge']) ? $featuredBadges[$i]['badge'] : $featuredBadges[$i - 1]['badge'];              
            }

            $i++;
        }

        return $badges;
    }
}