<?php

class MasterBadge_DataWriter_Badge extends XenForo_DataWriter
{
    const DATA_TITLE = 'phraseTitle';

/* Start auto-generated lines of code. */

    protected function _getFields()
    {
        return array(
            'xf_badge' => array(
                'badge_id' => array('type' => self::TYPE_UINT, 'autoIncrement' => true),
                'display_order' => array('type' => self::TYPE_UINT, 'required' => true, 'default' => 0),
                'trophy_count' => array('type' => self::TYPE_UINT, 'required' => true, 'default' => 0),
                'type' => array('type' => self::TYPE_STRING, 'default' => 'visible',
                    'allowedValues' => array('visible', 'step', 'hidden'),
                ),
                'create_date' => array('type' => self::TYPE_UINT, 'default' => XenForo_Application::$time)
            )
        );
    }

    protected function _getExistingData($data)
    {
        if (!$id = $this->_getExistingPrimaryKey($data, 'badge_id'))
        {
            return false;
        }

        return array('xf_badge' => $this->_getBadgeModel()->getBadgeById($id));
    }

    protected function _getUpdateCondition($tableName)
    {
        return 'badge_id = ' . $this->_db->quote($this->getExisting('badge_id'));
    }

/* End auto-generated lines of code. */

    protected function _postSave()
    {
        $badgeId = $this->get('badge_id');
        
        $titlePhrase = $this->getExtraData(self::DATA_TITLE);
        if ($titlePhrase !== null)
        {
            $this->_insertOrUpdateMasterPhrase(
                $this->_getTitlePhraseName($badgeId), $titlePhrase, ''
            );
        }

        $this->_rebuildCache();
        $this->rebuildCounters();
    }

    protected function _postDelete()
    {
        $badgeId = $this->get('badge_id');

        $this->_deleteMasterPhrase($this->_getTitlePhraseName($badgeId));

        $this->_db->update('xf_trophy', array(
            'badge_id' => 0
        ), 'badge_id = ' . $this->_db->quote($badgeId));

        $this->_rebuildCache();
        $this->rebuildCounters();
    }   

    public function rebuildCounters()
    {
        $this->_getBadgeModel()->updateTrophyCount($this->get('badge_id'));
    }

    protected function _getTitlePhraseName($id)
    {
        return $this->_getBadgeModel()->getBadgeTitlePhraseName($id);
    }

    protected function _rebuildCache()
    {
        // $this->_getBadgeModel()->rebuildBadgeTrophiesCache();
    }

    protected function _getBadgeModel()
    {
        return $this->getModelFromCache('MasterBadge_Model_Badge');
    }
}