<?php

class MasterBadge_DataWriter_Trophy extends XFCP_MasterBadge_DataWriter_Trophy
{
    const OPTION_UPLOAD_ICON = 'uploadIcon';

    const IMAGE_PREPARED = 'imagePrepared';
    const SIZE_ORIGINAL = -1;

    public static $imageQuality = 85;

    protected function _getFields()
    {
        $fields = parent::_getFields();

        $fields['xf_trophy']['trophy_key'] = array('type' => self::TYPE_STRING, 'default' => '',
            'verification' => array('$this', '_verifyTrophyKey'), 'requiredError' => 'MasterBadge_please_enter_valid_trophy_key');
        $fields['xf_trophy']['badge_id'] = array('type' => self::TYPE_UINT, 'default' => 0);
        $fields['xf_trophy']['trophy_order'] = array('type' => self::TYPE_UINT, 'default' => 1);
        $fields['xf_trophy']['icon_date'] = array('type' => self::TYPE_UINT, 'default' => 0);
        $fields['xf_trophy']['callback_class'] = array('type' => self::TYPE_STRING,  'maxLength' => 75, 'default' => '');
        $fields['xf_trophy']['callback_method'] = array('type' => self::TYPE_STRING,  'maxLength' => 50, 'default' => '');

        return $fields;
    }

    protected function _getDefaultOptions()
    {
        $options = parent::_getDefaultOptions();

        $options[self::OPTION_UPLOAD_ICON] = false;

        return $options;
    }

    protected function _verifyTrophyKey(&$trophyKey)
    {
        if ($trophyKey !== '')
        {
            if (preg_match('/[^a-zA-Z0-9_]/', $trophyKey))
            {
                $this->error(new XenForo_Phrase('please_enter_an_id_using_only_alphanumeric'), 'trophy_key');
                return false;
            }

            if (intval($trophyKey))
            {
                $this->error(new XenForo_Phrase('MasterBadge_trophy_key_contain_character'), 'trophy_key');
                return false;
            }

            if ($this->isInsert() || $trophyKey != $this->getExisting('trophy_key'))
            {
                $existing = $this->_getTrophyModel()->getTrophyByTrophyKey($trophyKey);
                if ($existing)
                {
                    $this->error(new XenForo_Phrase('MasterBadge_trophy_keys_must_be_unique'), 'trophy_key');
                    return false;
                }
            }
        }

        return true;
    }

    public function setImage(XenForo_Upload $upload) 
    {
        if (!$upload->isValid()) 
        {
            throw new XenForo_Exception($upload->getErrors(), true);
        }

        if (!$upload->isImage()) 
        {
            throw new XenForo_Exception(new XenForo_Phrase('uploaded_file_is_not_valid_image'), true);
        };

        $this->setExtraData(self::IMAGE_PREPARED, $this->_prepareImage($upload));
        $this->set('icon_date', XenForo_Application::$time);
    }

    protected function _prepareImage(XenForo_Upload $upload) 
    {
        $outputFiles = array();
        $fileName = $upload->getTempFile();
        $imageType = $upload->getImageInfoField('type');
        $outputType = $imageType;
        $width = $upload->getImageInfoField('width');
        $height = $upload->getImageInfoField('height');

        if (!$imageType || !$width || !$height)
        {
            $imageInfo = getimagesize($fileName);
            if (!$imageInfo)
            {
                throw new XenForo_Exception('Non-image passed in to applyBadgeIcon');
            }
            $width = $imageInfo[0];
            $height = $imageInfo[1];
            $imageType = $imageInfo[2];
        }

        if (!in_array($imageType, array(IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG))) 
        {
            throw new XenForo_Exception(new XenForo_Phrase('uploaded_file_is_not_valid_image'), true);
        }

        if (!XenForo_Image_Abstract::canResize($width, $height))
        {
            throw new XenForo_Exception(new XenForo_Phrase('uploaded_image_is_too_big'), true);
        }

        $imageSizes = $this->_getTrophyModel()->getTrophyIconSizes();
        reset($imageSizes);

        while (list($sizeCode, $maxDimensions) = each($imageSizes)) 
        {
            $newTempFile = tempnam(XenForo_Helper_File::getTempDir(), 'xfa');
            if (!$newTempFile)
            {
                return false;
            }

            if ($maxDimensions == self::SIZE_ORIGINAL) 
            {
                copy($fileName, $newTempFile);
            } 
            else 
            {
                $image = XenForo_Image_Abstract::createFromFile($fileName, $imageType);
                if (!$image) 
                {
                    continue;
                }
    
                //$image->thumbnail($maxDimensions, $maxDimensions);
                $image->thumbnailFixedShorterSide($maxDimensions);

                if ($image->getOrientation() != XenForo_Image_Abstract::ORIENTATION_SQUARE)
                {
                    $cropX = floor(($image->getWidth() - $maxDimensions) / 2);
                    $cropY = floor(($image->getHeight() - $maxDimensions) / 2);
                    $image->crop($cropX, $cropY, $maxDimensions, $maxDimensions);
                }
    
                $image->output($outputType, $newTempFile, self::$imageQuality);
                unset($image);
            }

            $outputFiles[$sizeCode] = $newTempFile;
        }

        if (count($outputFiles) != count($imageSizes)) 
        {
            foreach ($outputFiles AS $tempFile) 
            {
                if ($tempFile != $fileName) 
                {
                    @unlink($tempFile);
                }
            }
            
            throw new XenForo_Exception('Non-image passed in to _prepareImage');
        }
        
        return $outputFiles;
    }

    protected function _moveImages($uploaded) 
    {
        if (is_array($uploaded)) 
        {
            $data = $this->getMergedData();
            foreach ($uploaded as $sizeCode => $tempFile) 
            {
                $filePath = $this->_getTrophyModel()->getTrophyIconFilePath($data, $sizeCode);
                $directory = dirname($filePath);
 
                if (XenForo_Helper_File::createDirectory($directory, true) && is_writable($directory)) 
                {
                    if (file_exists($filePath)) 
                    {
                        @unlink($filePath);
                    }
                    
                    $success = XenForo_Helper_File::safeRename($tempFile, $filePath);
                    if ($success) 
                    {
                        XenForo_Helper_File::makeWritableByFtpUser($filePath);
                    }
                }
            }
        }
    }

    protected function _preSave()
    {
        if (isset($GLOBALS['MasterBadge_ControllerAdmin_Trophy'])) 
        {
            $GLOBALS['MasterBadge_ControllerAdmin_Trophy']->MasterBadge_actionSave($this);
        }

        return parent::_preSave();
    }

    protected function _postSave() 
    {
        $uploaded = $this->getExtraData(self::IMAGE_PREPARED);
        if ($uploaded) 
        {
            $this->_moveImages($uploaded);

            if ($this->isUpdate()) 
            {
                $existingData = $this->getMergedExistingData();
                foreach (array_keys($this->_getTrophyModel()->getTrophyIconSizes()) as $sizeCode) 
                {
                    $filePath = $this->_getTrophyModel()->getTrophyIconFilePath($existingData, $sizeCode);
                    if (!empty($filePath)) 
                    {
                        @unlink($filePath);
                    }
                }
            }
        }

        if ($this->getOption(self::OPTION_UPLOAD_ICON))
        {
            XenForo_Application::getSession()->set('uploadTrophyIcon', $this->getMergedData());
        }

        $this->_rebuildCache();
        $this->_getBadgeModel()->updateTrophyCount($this->get('badge_id'));

        return parent::_postSave();
    }
    
    protected function _postDelete() 
    {
        $existingData = $this->getMergedExistingData();
        foreach (array_keys($this->_getTrophyModel()->getTrophyIconSizes()) as $sizeCode) 
        {
            $filePath = $this->_getTrophyModel()->getTrophyIconFilePath($existingData, $sizeCode);
            if (!empty($filePath)) 
            {
                @unlink($filePath);
            }
        }

        $this->_rebuildCache();
        $this->_getBadgeModel()->updateTrophyCount($this->get('badge_id'));

        return parent::_postDelete();
    }

    protected function _rebuildCache()
    {
        // $this->_getBadgeModel()->rebuildBadgeTrophiesCache();
        $this->_getTrophyModel()->rebuildTrophiesCache();
    }

    protected function _getBadgeModel()
    {
        return $this->getModelFromCache('MasterBadge_Model_Badge');
    }
}