<?php

class MasterBadge_Helper_Level
{
    public static function increaseEach10($level)
    {
        $gap = XenForo_Application::getOptions()->MasterBadge_pointsPerLevel;

        $points = $x = 0;

        for ($i = 1; $i < $level; $i++) 
        {
            $x += ($i % 10 == 9);
        }

        for ($j = 0; $j <= $x; $j++) 
        { 
            $points += $j * 10 * $gap;
        }

        $points += ($level - ($x * 10)) * (($x + 1) * $gap);

        return $points;
    }

    public static function increaseEach($level)
    {
        $gap = XenForo_Application::getOptions()->MasterBadge_pointsPerLevel;

        $points = $level * $gap;

        for ($i = 1; $i < $level; $i++) 
        {
            $points += $i * $gap;
        }

        return $points;
    }

    public static function persistent($level)
    {
        $gap = XenForo_Application::getOptions()->MasterBadge_pointsPerLevel;

        return $level * $gap;
    }

    public static function getPointsByLevel($level)
    {
        if (!is_numeric($level))
        {
            return false;
        }

        if ($maxLevel = XenForo_Application::getOptions()->MasterBadge_maxLevel)
        {
            $level = $level > $maxLevel ? $maxLevel : $level;
        }

        $levelUp = XenForo_Application::getOptions()->MasterBadge_levelUp;

        switch ($levelUp) 
        {
            case 'persistent':
                $points = static::persistent($level);
                break;

            case 'increase1':
                $points = static::increaseEach($level);
                break;

            case 'increase10':
                $points = static::increaseEach10($level);
                break;
            
            default:
                $points = 0;
                break;
        }

        return $points;
    }

    public static function getLevelByPoints($points, $fromCache = true, $list = false)
    {
        if ($fromCache)
        {
            if ($level = static::getLevelByPointsFromCache($points))
            {
                return $level;
            }
        }

        $level = 0;
        $cache = array();

        if (!is_numeric($points) || $points < XenForo_Application::getOptions()->MasterBadge_pointsPerLevel)
        {
            return $level;
        }

        $_points = $points;

        for ($i = 0; $i <= $_points; $i++)
        {
            $nextLevelPoints = self::getPointsByLevel($i + 1);
            $currentLevelPoints = self::getPointsByLevel($i);

            $points -= ($nextLevelPoints - $currentLevelPoints);            

            if ($points < 0)
            {
                break;
            }

            // if ($list)
            // {
            //     // $cache[$currentLevelPoints] = $level;
            //     $cache[$level] = $currentLevelPoints;
            // }

            $level += 1;

            if ($list)
            {
                $cache[$level] = $nextLevelPoints;
            }
        }

        if ($maxLevel = XenForo_Application::getOptions()->MasterBadge_maxLevel)
        {
            return $level > $maxLevel ? $maxLevel : $level;
        }

        if ($list)
        {
            return $cache;
        }

        return $level;
    }

    public static function getLevelByPointsFromCache($points)
    {
        $listLevels = XenForo_Application::getSimpleCacheData('listLevels');

        if (!$listLevels || !is_array($listLevels) || !$points)
        {
            return false;
        }

        $max = max($listLevels);

        if ($points > $max)
        {
            static::rebuildLevelCache();
            return false;
        }

        if (($max / $points) < 2)
        {
            krsort($listLevels);
        }

        foreach ($listLevels as $level => $levelPoints) 
        {
            if ($points >= $levelPoints)
            {
                return $level;
            }
        }

        return false;
    }

    public static function rebuildLevelCache()
    {
        // Get Highest Level
        $db = XenForo_Application::getDb();
        $user = $db->fetchRow('SELECT * FROM xf_user ORDER BY trophy_points DESC LIMIT 1');
        
        if (!empty($user['trophy_points']))
        {
            $listLevels = static::getLevelByPoints($user['trophy_points'], false, true);

            if (!$listLevels || !is_array($listLevels))
            {
                return false;
            }

            // $max = max($listLevels);
            $max = key(array_slice($listLevels, -1, 1, true));

            $listLevels[$max + 1] = static::getPointsByLevel($max + 1);
            $listLevels = array_merge(array(0 => 0), $listLevels);

            XenForo_Application::setSimpleCacheData('listLevels', $listLevels);
        }
    }
}