<?php

class MasterBadge_Listener
{
    protected static $_trophyModel;
    protected static $_userModel;
    protected static $_badgeModel;

    public static function loadAccountController($class, array &$extend)
    {
        $extend[] = 'MasterBadge_ControllerPublic_Account';
    }

    public static function loadMemberController($class, array &$extend)
    {
        $extend[] = 'MasterBadge_ControllerPublic_Member';
    }

    public static function loadHelpController($class, array &$extend)
    {
        $extend[] = 'MasterBadge_ControllerPublic_Help';
    }

    public static function loadTrophyControllerAdmin($class, array &$extend)
    {
        $extend[] = 'MasterBadge_ControllerAdmin_Trophy';
    }

    public static function loadTrophyModel($class, array &$extend)
    {
        $extend[] = 'MasterBadge_Model_Trophy';
    }

    public static function loadUserModel($class, array &$extend)
    {
        $extend[] = 'MasterBadge_Model_User';
    }

    public static function loadTrophyDataWriter($class, array &$extend)
    {
        $extend[] = 'MasterBadge_DataWriter_Trophy';
    }

    public static function loadUserAlertHandler($class, array &$extend)
    {
        $extend[] = 'MasterBadge_AlertHandler_User';
    }

    public static function loadTrophyDeferred($class, array &$extend)
    {
        $extend[] = 'MasterBadge_Deferred_Badge';
    }

    public static function loadUserDataWriter($class, array &$extend)
    {
        $extend[] = 'MasterBadge_DataWriter_User';
    }

    public static function loadMemberRoute($class, array &$extend)
    {
        $extend[] = 'MasterBadge_Route_Prefix_Members';
    }

    public static function loadOptionDataWriter($class, array &$extend)
    {
        $extend[] = 'MasterBadge_DataWriter_Option';
    }

    public static function loadOptionModel($class, array &$extend)
    {
        $extend[] = 'MasterBadge_Model_Option';
    }

    public static function loadOptionControllerAdmin($class, array &$extend)
    {
        $extend[] = 'MasterBadge_ControllerAdmin_Option';
    }

    public static function initDependencies(XenForo_Dependencies_Abstract $dependencies, array $data)
    {        
        XenForo_Template_Helper_Core::$helperCallbacks += array(
            'trophyicon' => array('MasterBadge_Helper_Trophy', 'getTrophyIcon'),
            'level' => array('MasterBadge_Helper_Level', 'getLevelByPoints'),
            'levelpoints' => array('MasterBadge_Helper_Level', 'getPointsByLevel'),
            'userlevel' => array('MasterBadge_Core', 'getUserLevel'),
            'levelhtml' => array('MasterBadge_Core', 'getLevelHtml'),
            'userlevelhtml' => array('MasterBadge_Core', 'getUserLevelHtml'),            
        );

        if (XenForo_Application::getOptions()->userTitleLadderField == 'level')
        {
            XenForo_Template_Helper_Core::$helperCallbacks['usertitle'] = array('MasterBadge_Core', 'helperUserTitle');
        }
    }

    public static function criteriaUser($rule, array $data, array $user, &$returnValue)
    {
        switch ($rule)
        {
            case 'today':
                $today = XenForo_Locale::date(
                    XenForo_Application::$time, 'Y-m-d', null,
                    (isset($user['timezone']) ? $user['timezone'] : null)
                );

                if ($data['date'] === $today)
                {
                    $returnValue = true;
                }
            break;
        }
    }

    public static function forumListPostDispatch(XenForo_Controller $controller, $response, $controllerName, $action)
    {
        if (!($response instanceof XenForo_ControllerResponse_View))
        {
            return;
        }

        $options = XenForo_Application::getOptions();
        
        $response->params['badgeWidget'] = array();

        if ($options->MasterBadge_memberWidget['enabled'])
        {
            $userModel = self::_getUserModel();

            if ($options->MasterBadge_memberWidgetContent['topLevel'])
            {
                $conditions = array(
                    'user_state' => 'valid',
                    'is_banned' => 0
                );

                $fetchOptions = array(
                    'limit' => $options->MasterBadge_memberWidgetLimit,
                    'order' => 'trophy_points',
                    'direction' => 'DESC'
                );

                $response->params['badgeWidget']['users'] = $userModel->getUsers($conditions, $fetchOptions);
            }
        }

        if (XenForo_Visitor::getUserId() > 0 && $options->MasterBadge_widget['enabled'])
        {
            $response->params['progress'] = MasterBadge_Core::getUserLevelProgress(XenForo_Visitor::getInstance()->toArray());
        }
    }

    public static function controllerPostDispatch(XenForo_Controller $controller, $controllerResponse, $controllerName, $action)
    {
        if (!$controller instanceof XenForo_ControllerPublic_Abstract)
        {
            return;
        }

        $alertUsers = XenForo_Application::getSimpleCacheData('MasterBadge_alertUsers');

        if (!empty($alertUsers))
        {            
            XenForo_Application::defer('MasterBadge_Deferred_Alert', $alertUsers);
            XenForo_Application::setSimpleCacheData('MasterBadge_alertUsers', false);
        }
    }

    public static function navigationTabs(array &$extraTabs, $selectedTabId)
    {   
        if (XenForo_Application::getOptions()->MasterBadge_navigation)
        {
            $extraTabs['MasterBadge'] = array(
                'title' => new Xenforo_Phrase('MasterBadge_badges'),
                'href' => XenForo_Link::buildPublicLink('full:badges'),
                'MasterBadge' => $selectedTabId == 'MasterBadge' ? true : false,
                'linksTemplate' => 'MasterBadge_navigation_tabs',
                'position' => 'end'
            );      
        }
    }

    public static function visitorSetup(XenForo_Visitor &$visitor)
    {
        $visitor['badgeSlideOptions'] = self::_getBadgeModel()->getUserFeaturedBadgeSlideOptions($visitor->toArray());
    }

    protected static function _getTrophyModel()
    {
        if (!self::$_trophyModel)
        {
            self::$_trophyModel = XenForo_Model::create('XenForo_Model_Trophy');
        }

        return self::$_trophyModel;
    }

    protected static function _getBadgeModel()
    {
        if (!self::$_badgeModel)
        {
            self::$_badgeModel = XenForo_Model::create('MasterBadge_Model_Badge');
        }

        return self::$_badgeModel;
    }

    protected static function _getUserModel()
    {
        if (!self::$_userModel)
        {
            self::$_userModel = XenForo_Model::create('XenForo_Model_User');
        }

        return self::$_userModel;
    }
}