<?php

class MasterBadge_Model_Badge extends XenForo_Model
{
/* Start auto-generated lines of code. */

    public function getBadgeById($badgeId, array $fetchOptions = array())
    {
        $joinOptions = $this->prepareBadgeFetchOptions($fetchOptions);

        return $this->_getDb()->fetchRow('
            SELECT badge.*
                ' . $joinOptions['selectFields'] . '
            FROM xf_badge AS badge
                ' . $joinOptions['joinTables'] . '
            WHERE badge.badge_id = ?
        ', $badgeId);
    }

    public function getBadgesByIds($badgeIds, array $fetchOptions = array())
    {
        if (!$badgeIds)
        {
            return array();
        }

        $joinOptions = $this->prepareBadgeFetchOptions($fetchOptions);

        return $this->fetchAllKeyed('
            SELECT badge.*
                ' . $joinOptions['selectFields'] . '
            FROM xf_badge AS badge
                ' . $joinOptions['joinTables'] . '
            WHERE badge.badge_id IN (' . $this->_getDb()->quote($badgeIds) . ')
        ', 'badge_id');
    }

    public function getBadges(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareBadgeConditions($conditions, $fetchOptions);

        $orderClause = $this->prepareBadgeOrderOptions($fetchOptions, 'badge.display_order');
        $joinOptions = $this->prepareBadgeFetchOptions($fetchOptions);
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->fetchAllKeyed($this->limitQueryResults('
                SELECT badge.*
                    ' . $joinOptions['selectFields'] . '
                FROM xf_badge AS badge
                    ' . $joinOptions['joinTables'] . '
                WHERE ' . $whereClause . '
                ' . $orderClause . '
            ', $limitOptions['limit'], $limitOptions['offset']
        ), 'badge_id');
    }

    public function countBadges(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareBadgeConditions($conditions, $fetchOptions);
        $joinOptions = $this->prepareBadgeFetchOptions($fetchOptions);

        return $this->_getDb()->fetchOne('
            SELECT COUNT(*)
            FROM xf_badge AS badge
            ' . $joinOptions['joinTables'] . '
            WHERE ' . $whereClause
        );
    }

    public function prepareBadgeConditions(array $conditions = array(), array $fetchOptions = array())
    {
        $sqlConditions = array();
        $db = $this->_getDb();

        if (!empty($conditions['badge_id']))
        {
            if (is_array($conditions['badge_id']))
            {
                $sqlConditions[] = "badge.badge_id IN (" . $db->quote($conditions['badge_id']) . ")";
            }
            else
            {
                $sqlConditions[] = "badge.badge_id = " . $db->quote($conditions['badge_id']);
            }
        }

        if (!empty($conditions['tropy_count']) && is_array($conditions['tropy_count']))
        {
            $sqlConditions[] = $this->getCutOffCondition("badge.tropy_count", $conditions['tropy_count']);
        }

        if (!empty($conditions['type']) && $conditions['type'] !== 'any')
        {
            if (is_array($conditions['type']))
            {
                $sqlConditions[] = 'badge.type IN (' . $db->quote($conditions['type']) . ')';
            }
            else
            {
                $sqlConditions[] = 'badge.type = ' . $db->quote($conditions['type']);
            }
        }

        return $this->getConditionsForClause($sqlConditions);
    }

    public function prepareBadgeFetchOptions(array $fetchOptions = array())
    {
        $selectFields = '';
        $joinTables = '';

        return array(
            'selectFields' => $selectFields,
            'joinTables'   => $joinTables
        );
    }

    public function prepareBadgeOrderOptions(array $fetchOptions = array(), $defaultOrderSql = '')
    {
        $choices = array(
            'display_order' => 'badge.display_order',
            'trophy_order' => 'badge.display_order, trophy.trophy_order'
        );

        return $this->getOrderByClause($choices, $fetchOptions, $defaultOrderSql);
    }

    public function prepareBadges(array $badges, array $viewingUser = null)
    {
        $this->standardizeViewingUserReference($viewingUser);

        foreach ($badges AS &$badge)
        {
            $badge = $this->prepareBadge($badge, $viewingUser);
        }

        return $badges;
    }

    public function prepareBadge(array $badge, array $viewingUser = null)
    {
        $this->standardizeViewingUserReference($viewingUser);

        if (!empty($badge['badge_id']))
        {
            $badge['title'] = new XenForo_Phrase($this->getBadgeTitlePhraseName($badge['badge_id']));
            //$badge['canView'] = $this->canViewBadge($badge, $null, $viewingUser);
        }

        return $badge;
    }

/* End auto-generated lines of code. */

    public function getBadgeOptions($selectedBadgeId = 0)
    {
        $badges = $this->getBadges();

        $options = array();

        foreach ($badges as $badge)
        {
            if ($badge['badge_id'] !== $selectedBadgeId)
            {
                $options[] = array(
                    'label' => new XenForo_Phrase($this->getBadgeTitlePhraseName($badge['badge_id'])),
                    'value' => $badge['badge_id']
                );
            }
        }

        return $options;
    }

    public function getBadgeTitlePhraseName($badgeId)
    {
        return 'MasterBadge_badge_' . $badgeId . '_title';
    }

    public function getBadgeWithTrophiesForCache()
    {
        // if (XenForo_Application::getSimpleCacheData('badges'))
        // {
        //  $badges = XenForo_Application::getSimpleCacheData('badges');
        // }
        // else
        // {
        //  $badges = $this->rebuildBadgeTrophiesCache();
        // }
        $badges = $this->rebuildBadgeTrophiesCache();

        return $badges;
    }

    public function rebuildBadgeTrophiesCache()
    {
        $badges = $this->getBadgeWithTrophies();
        // XenForo_Application::setSimpleCacheData('badges', $badges);

        return $badges;
    }

    public function prepareBadgeForUser(array $badges, $user = null)
    {
        $this->standardizeViewingUserReference($user);
        $awarded = $this->_getTrophyModel()->getTrophiesForUserId($user['user_id']);

        foreach ($badges as $badgeId => &$badge) 
        {
            if ($badge['type'] == 'visible')
            {
                $badge['current'] = reset($badge['trophies']);
            }

            $trophyKeys = array_keys($badge['trophies']);
            $i = 0;

            foreach ($badge['trophies'] as $trophyId => &$trophy) 
            {
                if (in_array($trophyId, array_keys($awarded)))
                {
                    $trophy['award_date'] = $awarded[$trophyId]['award_date'];
                    $badge['current'] = array(
                        'trophy_id' => $trophyId,
                        'title' => $trophy['title'],
                        'description' => $trophy['description'],
                        'trophy_points' => $awarded[$trophyId]['trophy_points'],
                        'trophy_order' => $awarded[$trophyId]['trophy_order'],
                        'icon_date' => $awarded[$trophyId]['icon_date'],
                        'award_date' => $awarded[$trophyId]['award_date'],
                    );
                    if ($badge['type'] == 'step')
                    {
                        $next = !empty($trophyKeys[$i + 1]) ? $trophyKeys[$i + 1] : null;
                        if ($next)
                        {
                            $badge['next'] = $badge['trophies'][$next];
                        }
                    }
                }
                $i++;
            }
        }
        /*if (isset($badges[0]))
        {
            unset($badges[0]);
        }*/ 

        return $badges;
    }

    public function getBadgeWithTrophies()
    {
        $trophies = $this->fetchAllKeyed('
            SELECT badge.*, trophy.*
            FROM xf_trophy AS trophy
            LEFT JOIN xf_badge AS badge ON
                (badge.badge_id = trophy.badge_id)
            ORDER BY badge.display_order, trophy.trophy_order
        ', 'trophy_id');

        $badges = $this->_getDefaultBadges();

        $trophies = $this->_getTrophyModel()->prepareTrophies($trophies);

        foreach ($trophies as $trophyId => $trophy) 
        {
            $badges[$trophy['badge_id']]['badge_id'] = $trophy['badge_id'];
            $badges[$trophy['badge_id']]['title'] = new XenForo_Phrase($this->getBadgeTitlePhraseName($trophy['badge_id']));
            $badges[$trophy['badge_id']]['type'] = $trophy['type'];

            $badges[$trophy['badge_id']]['trophies'][$trophyId] = $trophy;
        }

        $badges[0]['type'] = 'hidden';
        if (empty($badges[0]['trophies']))
        {
            unset($badges[0]);
        }

        return $badges;
    }

    public function getAllBadgesWithTrophies(array $trophies = array())
    {
        $badges = $this->_getDefaultBadges();
        foreach ($this->getBadges() AS $id => $badge)
        {
            $badges[$id] = $badge;
        }

        if (!$trophies)
        {
            $trophies = $this->_getTrophyModel()->getAllTrophies();
            $trophies = $this->_getTrophyModel()->prepareTrophies($trophies);
        }

        usort($trophies, function($a, $b) {
            return $a['trophy_order'] - $b['trophy_order'];
        });

        foreach ($trophies AS $trophyId => $trophy)
        {
            $badges[$trophy['badge_id']]['trophies'][$trophyId] = $trophy;
        }

        return $badges;
    }

    public function prepareBadgeTrophiesForList(array $badges, &$totalTrophies = 0)
    {
        foreach ($badges AS $badgeId => &$badge)
        {
            $badge = $this->prepareBadge($badge);

            if (!empty($badge['trophies']))
            {
                $badge['trophies'] = $this->_getTrophyModel()->prepareTrophies($badge['trophies']);

                $totalTrophies += count($badge['trophies']);
            }
        }

        return $badges;
    }

    public function buildCurrentBadgeTropy(array $trophies)
    {
        $badges = array();

        foreach ($trophies as $trophyId => $trophy) 
        {
            $key = $trophy['badge_id'] ? $trophy['badge_id'] : $trophyId * -1;

            if (isset($badges[$key]))
            {
                if ($badges[$key]['award_date'] < $trophy['award_date'])
                {
                    $badges[$key] = $trophy;
                }
            }
            else
            {
                $badges[$key] = $trophy;
            }
        }

        return $badges;
    }

    public function updateTrophyCount($badgeId, $adjust = null)
    {
        if (!$badgeId)
        {
            return false;
        }

        $db = $this->_getDb();
        XenForo_Db::beginTransaction($db);

        if ($adjust === null)
        {
            $trophyCount = $db->fetchOne('
                SELECT COUNT(*)
                FROM xf_trophy
                WHERE badge_id = ?
            ', $badgeId);

            $db->query('
                UPDATE xf_badge
                SET trophy_count = ?
                WHERE badge_id = ?
            ', array($trophyCount, $badgeId));
        }
        else
        {
            $db->query('
                UPDATE xf_badge
                SET trophy_count = trophy_count + ' . $adjust . '
                WHERE badge_id = ?
            ', $badgeId);
        }

        XenForo_Db::commit($db);
    }

    protected function _getDefaultBadges()
    {
        return array(
            0 => array(
                'badge_id' => 0,
                'trophies' => array(),
            )
        );
    }

    public function limitFeaturedBadges(array $user, $badges)
    {
        if (!$badges)
        {
            return array();
        }

        $threshold = MasterBadge_Core::getFeaturedBadges(MasterBadge_Core::getUserLevel($user));
        return array_slice($badges, 0, $threshold, true);
    }

    public function getUserFeaturedBadges(array $user, $key = 'favorite_badge')
    {
        if (!isset($user[$key]))
        {
            return array();
        }

        $trophyModel = $this->_getTrophyModel();

        if (!empty($user['auto_feature_badge']))
        {
            $favoriteBadges = @unserialize($user['awarded_trophy']);

            if (empty($favoriteBadges))
            {
                $trophies = $trophyModel->getTrophiesForUserId($user['user_id']);
                $favoriteBadges = array_keys($trophies);

                $dw = XenForo_DataWriter::create('XenForo_DataWriter_User');
                $dw->setExistingData($user);
                $dw->set('awarded_trophy', $favoriteBadges);
                $dw->save();
            }
        }
        else
        {
            $favoriteBadges = @unserialize($user[$key]);
        }

        $favoriteBadges = is_array($favoriteBadges) ? $favoriteBadges : array();
        
        $badges = array();

        foreach ($favoriteBadges as $trophyId)
        {
            if ($trophy = $trophyModel->getTrophyFromCache($trophyId))
            {
                $badges[$trophyId] = $trophy;
            }
        }

        $badges = $this->limitFeaturedBadges($user, $badges);

        return $badges;
    }

    public function getUserFeaturedBadgeSlideOptions(array $user)
    {
        $badgeSlideOptions = isset($user['feature_badge_slide_options']) 
            ? @unserialize($user['feature_badge_slide_options']) 
            : array();

        if (isset($badgeSlideOptions['autoplay']))
        {
            $badgeSlideOptions['autoplay'] = (bool) $badgeSlideOptions['autoplay'];
        }

        return $badgeSlideOptions;
    }

    public function getBadgeTypeStats()
    {
        $stats = $this->_getDb()->fetchAll('
            SELECT type, SUM(trophy_count) as total 
            FROM xf_badge 
            GROUP BY type 
        ');

        if (!$stats)
        {
            $stats = array();
        }

        $hidden = $this->_getDb()->fetchOne('
            SELECT COUNT(*)
            FROM xf_trophy
            WHERE badge_id = ?
        ', 0);

        $stats[] = array('type' => 'hidden', 'total' => $hidden);

        return $stats;
    }

    public function canViewBadge(array $badge, &$errorPhraseKey = '', array $viewingUser = null)
    {
        $this->standardizeViewingUserReference($viewingUser);

        if (!$viewingUser['user_id'])
        {
            return false;
        }

        if ($badge['type'] == 'visible')
        {
            return true;
        }

        $trophyModel = $this->_getTrophyModel();
        $trophies = $trophyModel->getTrophyByBadgeId($badge['badge_id']);
        $awarded = $trophyModel->getTrophiesForUserId($viewingUser['user_id']);

        foreach ($trophies as $trophy) 
        {
            if (in_array($trophy['trophy_id'], array_keys($awarded)))
            {
                return true;
            }
        }

        return false;
    }

    protected function _assertBadgesMode($mode)
    {
        return in_array($mode, array('visible', 'step', 'hidden'));
    }

    protected function _getTrophyModel()
    {
        return $this->getModelFromCache('XenForo_Model_Trophy');
    }
}