<?php

class MasterBadge_Model_Trophy extends XFCP_MasterBadge_Model_Trophy
{
    /** 
     * @var array
     */
    public static $ignoredCriteria = array('user_groups', 'not_user_groups', 'style', 'language');

    /** 
     * @var array
     */
    protected static $alertUsers = array();

    public function getTrophiesByIds($trophyIds)
    {
        if (!$trophyIds)
        {
            return array();
        }

        return $this->fetchAllKeyed('
            SELECT item.*
            FROM xf_trophy
            WHERE trophy_id IN (' . $this->_getDb()->quote($trophyIds) . ')
        ', 'trophy_id');
    }

    public function getTrophyByBadgeId($badgeId, array $fetchOptions = array())
    {
        return $this->fetchAllKeyed('
            SELECT *
            FROM xf_trophy
            WHERE badge_id = ' . $badgeId . '
        ', 'trophy_id');
    }

    public function getUsersByTrophyId($trophyId, array $fetchOptions = array())
    {
        $orderClause = $this->prepareTrophyUserOrderOptions($fetchOptions, 'user.username');
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

        return $this->fetchAllKeyed($this->limitQueryResults('
            SELECT user.*, user_trophy.*
            FROM xf_user AS user
            INNER JOIN xf_user_trophy AS user_trophy
                ON (user.user_id = user_trophy.user_id)
            WHERE user_trophy.trophy_id = ?
            ' . $orderClause . '
            ', $limitOptions['limit'], $limitOptions['offset']
        ), 'user_id', $trophyId);
    }

    public function countTrophyUsers(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareTrophyUserConditions($conditions, $fetchOptions);

        return $this->_getDb()->fetchOne('
            SELECT COUNT(*)
            FROM xf_user_trophy AS user_trophy
            WHERE ' . $whereClause
        );
    }

    public function getTrophyByTrophyKey($trophyKey, array $fetchOptions = array())
    {
        return $this->_getDb()->fetchRow('
            SELECT *
            FROM xf_trophy
            WHERE trophy_key = ?
        ', $trophyKey);
    }

    public function getLatestAwarded(array $conditions = array(), array $fetchOptions = array())
    {
        $orderClause = $this->prepareTrophyUserOrderOptions($fetchOptions, 'user_trophy.award_date DESC');
        $limitOptions = $this->prepareLimitFetchOptions($fetchOptions);
        $whereClause = $this->prepareTrophyUserConditions($conditions, $fetchOptions);

        $extraClause = isset($fetchOptions['extra']) ? ' ' . $fetchOptions['extra'] : '';

        return $this->_getDb()->fetchAll($this->limitQueryResults('
            SELECT trophy.*,
                user.*,
                user_trophy.award_date
            FROM xf_user_trophy AS user_trophy
            LEFT JOIN xf_trophy AS trophy ON (trophy.trophy_id = user_trophy.trophy_id)
            LEFT JOIN xf_user AS user ON (user.user_id = user_trophy.user_id)
            WHERE ' . $whereClause . '
            ' . $extraClause . '
            ' . $orderClause . '
            ', $limitOptions['limit'], $limitOptions['offset']
        ));
    }

    public function filterGraphData(array $data, $start, $end)
    {
        $plot = array(
            'x' => array(),
            'total' => array(),
        );

        while ($start <= $end)
        {
            $plot['x'][$start] = $start * 1000; // Milisecond
            $lower = $start;
            $higher = $start += 3600 * 24;

            $plot['total'][$start] = 0;

            foreach ($data as $item) 
            {
                if (($lower < $item['award_date']) && ($item['award_date'] <= $higher))
                {
                    $plot['total'][$start]++;
                }
            }
        }

        return $plot;
    }

    public function prepareGraphData(array $data)
    {
        foreach ($data as $key => &$_data) 
        {
            $_data = array_values($_data);
        }

        return $data;
    }

    public function getLatestAwardedCount($start, $end)
    {
        return $this->fetchAllKeyed('
            SELECT COUNT(*) AS total,
                trophy.badge_id
            FROM xf_user_trophy AS user_trophy
            LEFT JOIN xf_trophy AS trophy ON (trophy.trophy_id = user_trophy.trophy_id)
            WHERE user_trophy.award_date BETWEEN ? AND ?
            GROUP BY trophy.badge_id
        ', 'badge_id', array($start, $end));
    }

    public function countBadgeUsers(array $conditions = array(), array $fetchOptions = array())
    {
        $whereClause = $this->prepareTrophyUserConditions($conditions, $fetchOptions);

        return $this->fetchAllKeyed('
            SELECT COUNT(DISTINCT(user_trophy.user_id)) AS total, trophy.badge_id
            FROM xf_user_trophy AS user_trophy
                LEFT JOIN xf_trophy AS trophy ON (trophy.trophy_id = user_trophy.trophy_id)
            WHERE ' . $whereClause . '
            GROUP BY trophy.badge_id
        ', 'badge_id');
    }

    public function prepareTrophyUserConditions(array $conditions, array &$fetchOptions)
    {
        $db = $this->_getDb();
        $sqlConditions = array();

        if (!empty($conditions['user_id']))
        {
            if (is_array($conditions['user_id']))
            {
                $sqlConditions[] = 'user_trophy.user_id IN(' . $db->quote($conditions['user_id']) . ')';
            }
            else
            {
                $sqlConditions[] = 'user_trophy.user_id = ' . $db->quote($conditions['user_id']);
            }
        }

        if (!empty($conditions['trophy_id']))
        {
            if (is_array($conditions['trophy_id']))
            {
                $sqlConditions[] = 'user_trophy.trophy_id IN (' . $db->quote($conditions['trophy_id']) . ')';
            }
            else
            {
                $sqlConditions[] = 'user_trophy.trophy_id = ' . $db->quote($conditions['trophy_id']);
            }
        }

        if (!empty($conditions['badge_id']))
        {
            if (is_array($conditions['badge_id']))
            {
                $sqlConditions[] = 'trophy.badge_id IN (' . $db->quote($conditions['badge_id']) . ')';
            }
            else
            {
                $sqlConditions[] = 'trophy.badge_id = ' . $db->quote($conditions['badge_id']);
            }
        }

        if (!empty($conditions['award_date']) && is_array($conditions['award_date']))
        {
            $sqlConditions[] = $this->getCutOffCondition("user_trophy.award_date", $conditions['award_date']);
        }

        return $this->getConditionsForClause($sqlConditions);
    }

    public function prepareTrophyUserOrderOptions(array &$fetchOptions, $defaultOrderSql = '')
    {
        $choices = array(
            'username' => 'user.username',
            'award_date' => 'user_trophy.award_date'
        );
        return $this->getOrderByClause($choices, $fetchOptions, $defaultOrderSql);
    }

    public function getTrophyIconSizes()
    {
        static $imageSizes = false;
        
        if ($imageSizes === false) 
        {
            $imageSizes = array(
                'l' => MasterBadge_Const::ICON_SIZE_LARGE,
                'm' => MasterBadge_Const::ICON_SIZE_MEDIUM,
                's' => MasterBadge_Const::ICON_SIZE_SMALL,
            );
        }
        
        return $imageSizes;
    }

    public function getTrophyIconFilePath(array $trophy, $size = 'm', $externalDataPath = null)
    {
        if ($externalDataPath === null)
        {
            $externalDataPath = XenForo_Helper_File::getExternalDataPath();
        }

        return sprintf('%s/trophies/%d/%d_%d_%s.png',
            $externalDataPath,
            floor($trophy['trophy_id'] / 1000),
            $trophy['trophy_id'],
            $trophy['icon_date'],
            $size
        );
    }

    public function getTrophiesXml(array $trophyIds)
    {
        $trophies = $this->fetchAllKeyed('
            SELECT xf_trophy.*,
                xf_badge.*
            FROM xf_trophy
            LEFT JOIN xf_badge ON
                (xf_badge.badge_id = xf_trophy.badge_id)
            WHERE xf_trophy.trophy_id IN (' . $this->_getDb()->quote($trophyIds) . ')
            ORDER BY xf_badge.display_order, xf_trophy.trophy_order
        ', 'trophy_id');

        $document = new DOMDocument('1.0', 'utf-8');
        $document->formatOutput = true;

        $rootNode = $document->createElement('trophies_export');
        $document->appendChild($rootNode);

        $trophiesNode = $document->createElement('trophies');
        $badges = array();
        foreach ($trophies AS $trophy)
        {
            $trophyNode = $document->createElement('trophy');

            if ($trophy['badge_id'])
            {
                $badges[$trophy['badge_id']] = array(
                    'display_order' => $trophy['display_order'],
                    'type' => $trophy['type']
                );
                $trophyNode->setAttribute('badge_id', $trophy['badge_id']);
            }

            $trophyNode->setAttribute('title', new XenForo_Phrase($this->getTrophyTitlePhraseName($trophy['trophy_id'])));
            $trophyNode->setAttribute('description', new XenForo_Phrase($this->getTrophyDescriptionPhraseName($trophy['trophy_id'])));
            $trophyNode->setAttribute('trophy_key', $trophy['trophy_key']);
            $trophyNode->setAttribute('trophy_points', $trophy['trophy_points']);
            $trophyNode->setAttribute('trophy_order', $trophy['trophy_order']);
            $trophyNode->setAttribute('callback_class', $trophy['callback_class']);
            $trophyNode->setAttribute('callback_method', $trophy['callback_method']);

            $trophyCriteriaNode = $document->createElement('user_criteria');

            //$trophyCriteriaNode->appendChild(XenForo_Helper_DevelopmentXml::createDomCdataSection($document, $trophy['user_criteria']));

            foreach (unserialize($trophy['user_criteria']) AS $rule)
            {
                $ruleNode = $document->createElement('rule');
                $ruleNode->appendChild(XenForo_Helper_DevelopmentXml::createDomCdataSection($document, serialize($rule['data'])));
                $ruleNode->setAttribute('name', $rule['rule']);

                $trophyCriteriaNode->appendChild($ruleNode);
            }

            $trophyNode->appendChild($trophyCriteriaNode);

            $trophiesNode->appendChild($trophyNode);
        }

        $badgesNode = $document->createElement('badges');
        foreach ($badges AS $badgeId => $badge)
        {
            if ($badgeId)
            {
                $badgeNode = $document->createElement('badge');
                $badgeNode->setAttribute('id', $badgeId);
                $badgeNode->setAttribute('title', new XenForo_Phrase($this->_getBadgeModel()->getBadgeTitlePhraseName($badgeId)));
                $badgeNode->setAttribute('display_order', $badge['display_order']);
                $badgeNode->setAttribute('type', $badge['type']);

                $badgesNode->appendChild($badgeNode);
            }
        }

        $rootNode->appendChild($badgesNode);
        $rootNode->appendChild($trophiesNode);

        return $document;
    }

    public function getTrophyDataFromXml(SimpleXMLElement $document, array $existingBadgeOptions, &$errors = array())
    {
        if ($document->getName() != 'trophies_export')
        {
            throw new XenForo_Exception(new XenForo_Phrase('MasterBadge_provided_file_is_not_valid_trophies_xml'), true);
        }

        $categoryMap = array();
        $badges = array();
        $badgeOptions = array();

        foreach ($document->badges->badge AS $badge)
        {
            $existingId = array_search((string)$badge['title'], $existingBadgeOptions);
            if ($existingId !== false)
            {
                $categoryMap[(int)$badge['id']] = $existingId;
            }
            else
            {
                $id = (int)$badge['id'] * -1;
                $categoryMap[(int)$badge['id']] = $id;

                $badges[$id] = array(
                    'badge_id' => $id,
                    'display_order' => (int)$badge['display_order'],
                    'title' => (string)$badge['title'],
                    'type' => (string)$badge['type']
                );

                $badgeOptions[$id] = (string)$badge['title'];
            }
        }

        $trophies = array();
        $rules = array();
        $i = 0;

        foreach ($document->trophies->trophy AS $trophy)
        {
            $criteria = array();

            if ($trophy->user_criteria)
            {
                foreach ($trophy->user_criteria->rule as $rule) 
                {
                    $ruleName = (string)$rule['name'][0];

                    // Ignore User Groups criteria because the user_group_id may inconsistent for each board
                    if (!in_array($ruleName, static::$ignoredCriteria))
                    {                       
                        $criteria[$ruleName] = XenForo_Helper_Criteria::unserializeCriteria(XenForo_Helper_DevelopmentXml::processSimpleXmlCdata($rule));

                        $rules[$i][] = $ruleName;
                    }
                }
            }
            else
            {
                $spriteParams = $this->getDefaultSmilieSpriteParams();
            }

            if ((int)$trophy['badge_id'])
            {
                $badgeId = $categoryMap[(int)$trophy['badge_id']];
            }
            else
            {
                $badgeId = 0;
            }

            $trophies[$i] = array(
                'title' => (string)$trophy['title'],
                'description' => (string)$trophy['description'],
                'trophy_key' => (string)$trophy['trophy_key'],
                'trophy_points' => (int)$trophy['trophy_points'],
                'trophy_order' => (int)$trophy['trophy_order'],
                'callback_class' => (string)$trophy['callback_class'],
                'callback_method' => (string)$trophy['callback_method'],
                'badge_id' => $badgeId,
                'userCriteria' => $criteria
            );

            $i++;
        }

        return array(
            'rules' => $rules,
            'trophies' => $trophies,
            'newBadges' => $badges,
            'newBadgeOptions' => $badgeOptions
        );
    }

    public function massImportTrophies(array $trophies, array $badges, &$errors = array())
    {
        $db = $this->_getDb();

        $categoryMap = array();

        // get existing badges to avoid duplication - just in case
        $existingBadges = $this->_getExistingBadgesIdsByTitle();

        XenForo_Db::beginTransaction();

        foreach ($badges AS $badgeId => $badge)
        {
            $import = false;

            foreach ($trophies AS $trophy)
            {
                if ($trophy['badge_id'] == $badgeId)
                {
                    // only import categories that contain imported trophies
                    $import = true;
                    break;
                }
            }

            if ($import)
            {
                if (isset($existingBadges[$badge['title']]))
                {
                    // an existing badge has the title of the incoming badge, so use it

                    $categoryMap[$badgeId] = $existingBadges[$badge['title']];
                }
                else
                {
                    $dw = XenForo_DataWriter::create('MasterBadge_DataWriter_Badge');
                    $dw->set('display_order', $badge['display_order']);
                    $dw->set('type', $badge['type']);
                    $dw->setExtraData(MasterBadge_DataWriter_Badge::DATA_TITLE, $badge['title']);

                    if ($dwErrors = $dw->getErrors())
                    {
                        foreach ($dwErrors AS $field => $error)
                        {
                            $errors[$field . '__' . $badgeId] = $error;
                        }

                        XenForo_Db::rollback();
                        return;

                    }
                    else
                    {
                        $dw->save();
                        $categoryMap[$badgeId] = $dw->get('badge_id');
                    }
                }
            }
        }

        $dataWriters = array();

        foreach ($trophies AS $trophyId => $trophy)
        {
            $dw = XenForo_DataWriter::create('XenForo_DataWriter_Trophy');

            $dw->setExtraData(XenForo_DataWriter_Trophy::DATA_TITLE, $trophy['title']);
            $dw->setExtraData(XenForo_DataWriter_Trophy::DATA_DESCRIPTION, $trophy['description']);
            $dw->set('trophy_points', $trophy['trophy_points']);
            $dw->set('trophy_order', $trophy['trophy_order']);
            $dw->set('trophy_key', $trophy['trophy_key']);
            $dw->set('callback_class', $trophy['callback_class']);
            $dw->set('callback_method', $trophy['callback_method']);
            $dw->set('user_criteria', $trophy['user_criteria']);

            if ($trophy['badge_id'])
            {
                if ($trophy['badge_id'] < 0)
                {
                    $trophy['badge_id'] = $categoryMap[$trophy['badge_id']];
                }
            }

            $dw->set('badge_id', $trophy['badge_id']);

            if ($dwErrors = $dw->getErrors())
            {
                foreach ($dwErrors AS $field => $error)
                {
                    $errors[$field . '__' . $trophyId] = $error;
                }
            }
            else
            {
                $dataWriters[] = $dw;
            }
        }

        if (empty($errors))
        {
            foreach ($dataWriters AS $dw)
            {
                $dw->save();
            }

            XenForo_Db::commit();
        }
        else
        {
            XenForo_Db::rollback();
        }
    }

    protected function _getExistingBadgesIdsByTitle()
    {
        return $this->_getDb()->fetchPairs("
            SELECT p.phrase_text, b.badge_id
            FROM xf_badge AS b
            INNER JOIN xf_phrase AS p ON
                (p.language_id = 0 AND p.title = CONCAT('MasterBadge_badge_', b.badge_id, '_title'))
            ORDER BY b.display_order
        ");
    }

    public function deleteTrophyIcon($trophy)
    {
        $dw = XenForo_DataWriter::create('XenForo_DataWriter_Trophy');
        $dw->setExistingData($trophy);
        $dw->set('icon_date', 0);
        $dw->save();

        if (!is_array($trophy))
        {
            $trophy = $dw->getMergedData();
        }

        foreach (array_keys($this->getTrophyIconSizes()) as $sizeCode) 
        {
            $filePath = $this->getTrophyIconFilePath($trophy, $sizeCode);
            if (!empty($filePath)) 
            {
                @unlink($filePath);
            }
        }
    }

    public function massUpdateTrophyOrder(array $order)
    {
        $sqlOrder = '';
        $sqlParent = '';

        $db = $this->_getDb();

        foreach ($order AS $displayOrder => $data)
        {
            $trophyId = $db->quote((int)$data[0]);

            $sqlParent .= "WHEN $trophyId THEN " . $db->quote((int)$data[1]) . "\n";

            $sqlOrder .= "WHEN $trophyId THEN " . $db->quote((int)$displayOrder * 10) . "\n";
        }

        $db->query('
            UPDATE xf_trophy SET
            trophy_order = CASE trophy_id
            ' . $sqlOrder . '
                ELSE 0 END,
            badge_id = CASE trophy_id
            ' . $sqlParent . '
                ELSE 0 END
        ');
    }

    public function getTrophiesFromCache($prepare = true)
    {
        if (XenForo_Application::getSimpleCacheData('trophies'))
        {
            $trophies = XenForo_Application::getSimpleCacheData('trophies');
        }
        else
        {
            $trophies = $this->rebuildTrophiesCache();
        }

        if ($prepare)
        {
            $trophies = $this->prepareTrophies($trophies);
        }

        return $trophies;
    }

    public function getTrophyFromCache($trophyId, $prepare = true)
    {
        $trophies = $this->getTrophiesFromCache($prepare);

        return !empty($trophies[$trophyId]) ? $trophies[$trophyId] : false;
    }

    public function rebuildTrophiesCache()
    {
        $trophies = $this->getAllTrophies();
        XenForo_Application::setSimpleCacheData('trophies', $trophies);

        return $trophies;
    }

    public function updateTrophiesForUser(array $user, array $userTrophies = null, array $trophies = null)
    {
        $awarded = parent::updateTrophiesForUser($user, $userTrophies, $trophies);

        if ($trophies === null)
        {
            $trophies = $this->getAllTrophies();
        }
        if (!$trophies)
        {
            return 0;
        }

        if ($userTrophies === null)
        {
            $userTrophies = $this->getTrophiesForUserId($user['user_id']);
        }

        foreach ($trophies AS $trophy)
        {
            if (isset($userTrophies[$trophy['trophy_id']]))
            {
                continue;
            }

            if ($trophy['callback_class'] && $trophy['callback_method'])
            {
                if (class_exists($trophy['callback_class']) && is_callable(array($trophy['callback_class'], $trophy['callback_method'])))
                {
                    try
                    {
                        $callback = call_user_func(
                            array($trophy['callback_class'], $trophy['callback_method']),
                            $user,
                            $trophy
                        );

                        if ($callback === true)
                        {
                            $this->awardUserTrophy($user, $user['username'], $trophy);
                            $awarded++;
                        }
                    }
                    catch (Exception $e)
                    {
                        XenForo_Error::logException($e, false, 'Trophy callback error: ');
                    }
                }
            }
        }

        return $awarded;
    }

    public function awardUserTrophy(array $user, $username, array $trophy, $awardDate = null)
    {        
        parent::awardUserTrophy($user, $username, $trophy, $awardDate);

        if (!empty($user['auto_feature_badge']))
        {
            $trophies = $this->getTrophiesForUserId($user['user_id']);

            $dw = XenForo_DataWriter::create('XenForo_DataWriter_User');
            $dw->setExistingData($user);
            $dw->set('awarded_trophy', array_keys($trophies));
            $dw->save();
        }

        if (XenForo_Application::getOptions()->MasterBadge_alertLevelUp && XenForo_Model_Alert::userReceivesAlert($user, 'user', 'trophy'))
        {
            $this->queueAlert($user, $username);
        }

        XenForo_CodeEvent::fire('trophy_award', array($user, $trophy), $username);
    }

    public function queueAlert(array $user, $username)
    {
        if (!isset(static::$alertUsers[$user['user_id']]))
        {
            $user['level'] = MasterBadge_Core::getUserLevel($user);
            $user['alert_username'] = $username;
            static::$alertUsers[$user['user_id']] = $user;
        }
        else
        {
            static::$alertUsers[$user['user_id']]['level'] = MasterBadge_Core::getUserLevel($user);
        }

        XenForo_Application::setSimpleCacheData('MasterBadge_alertUsers', static::$alertUsers);
    }

    public function prepareTrophyForUser(array $trophy, $user = null)
    {
        $this->standardizeViewingUserReference($user);
        $awarded = $this->getTrophiesForUserId($user['user_id']);

        $trophy['award_date'] = false;

        if (isset($awarded[$trophy['trophy_id']]))
        {
            $trophy['award_date'] = $awarded[$trophy['trophy_id']]['award_date'];
        }

        return $trophy;
    }

    public function getCurrentBadgeTropy($badgeId, $user = null)
    {
        $badgeModel = $this->_getBadgeModel();

        $badges = $badgeModel->getBadgeWithTrophiesForCache();
        $badges = $badgeModel->prepareBadgeForUser($badges, $user);

        return (!empty($badges[$badgeId]) ? $badges[$badgeId] : false);
    }   

    public function removeUserTrophy($userId, $trophyId)
    {
        $userId = intval($userId);
        $trophyId = intval($trophyId);

        $db = $this->_getDb();

        $user = $this->getModelFromCache('XenForo_Model_User')->getUserById($userId);

        if (isset($user['favorite_badge']))
        {           
            if ($favoriteBadges = @unserialize($user['favorite_badge']))
            {
                if (in_array($trophyId, $favoriteBadges))
                {
                    $favoriteBadges = array_diff_key($favoriteBadges, array($trophyId));

                    $db->update('xf_user',
                        array('favorite_badge' => serialize($favoriteBadges)),
                        'user_id = ' . $db->quote($userId)
                    );
                }
            }

            $db->delete('xf_user_trophy', 
                'user_id = ' . $db->quote($userId) . ' AND trophy_id = ' . $db->quote($trophyId)
            );

            $this->updateTrophyPointsForUser($userId);
        }       
    }

    protected function _getBadgeModel()
    {
        return $this->getModelFromCache('MasterBadge_Model_Badge');
    }
}