<?php

class MasterBadge_Route_Prefix_Members extends XFCP_MasterBadge_Route_Prefix_Members
{
    /**
     * Resolve sub route i.e: members/{name}.{id}/badges/{other_id}/{action}
     * to correct controller and action. {other_id} is required to make sure this work.
     */ 
    public function match($routePath, Zend_Controller_Request_Http $request, XenForo_Router $router)
    {
        /* @var XenForo_RouteMatch */
        $routeMatch = parent::match($routePath, $request, $router);         

        $route = MasterBadge_Route::getSubRoute($routeMatch->getAction());

        if (is_array($route))
        {                               
            $parts = explode('/', $routePath, 3);
            $params = end($parts);

            $intParamName = isset($route['intId']) ? $route['intId'] : '';
            $stringParamName = isset($route['stringId']) ? $route['stringId'] : '';

            $action = $router->resolveActionWithIntegerOrStringParam($params, $request, $intParamName, $stringParamName);
            
            if (($intParamName && $request->getParam($intParamName))
                || ($stringParamName && $request->getParam($stringParamName))
            )
            {                
                if (isset($route['controller']))
                {
                    $routeMatch->setControllerName($route['controller']);
                    $routeMatch->setAction($action);
                }
            }
            // Because we use resolveActionWithIntegerOrStringParam so we need to make sure the data is existed
            /*else
            {
                if (isset($route['controller']))
                {
                    $routeMatch->setControllerName($route['controller']);
                    $routeMatch->setAction($params);
                }
            }*/
        }

        return $routeMatch;
    }

    /**
     * Build sub route link i.e: members/{name}.{id}/badges/{other_id}/{action}
     * {other_id} is required to make sure this work.
     */ 
    public function buildLink($originalPrefix, $outputPrefix, $action, $extension, $data, array &$extraParams)
    {
        $link = parent::buildLink($originalPrefix, $outputPrefix, $action, $extension, $data, $extraParams);        

        $route = MasterBadge_Route::getSubRoute($action);

        if (is_array($route))
        {            
            $_link = '';
             
            if (isset($route['intId']))
            {
                if (!empty($extraParams[$route['intId']]))
                {
                    $title = isset($extraParams[$route['title']]) 
                        ? $extraParams[$route['title']]
                        : '';


                    $_link = XenForo_Link::buildIntegerAndTitleUrlComponent($extraParams[$route['intId']], $title);
                    
                    if ($title)
                    {
                        unset($extraParams[$route['title']]);
                    }

                    unset($extraParams[$route['intId']]);
                }
            }
            else if (isset($route['stringId']))
            {
                if (isset($extraParams[$route['stringId']]))
                {
                    $_link = $extraParams[$route['stringId']];
                    unset($extraParams[$route['stringId']]);
                }
            }

            if ($_link && !$this->_checkSlash($_link))
            {   
                $_link .= '/';
            }

            if ($this->_checkSlash($link))
            {
                $link .= $_link;
            }
            else
            {
                $link .= '/' . $_link;
            }
        }        

        return $link;
    }

    protected function _checkSlash($text)
    {
        if (substr($text, -1) === '/')
        {
            return true;
        }

        return false;
    }
}