<?php

class MasterBadge_Widget
{
    protected static $_templateModificationModel;
    protected static $_userModel;

    public static function levelWidgetModification(array $matches)
    {
        $template = static::findAndReplace($matches[0], '<xen:include template="MasterBadge_widget" />', XenForo_Application::getOptions()->MasterBadge_widget);

        return $template;
    }

    public static function memberWidgetModification(array $matches)
    {
        $template = static::findAndReplace($matches[0], '<xen:include template="MasterBadge_member_widget" />', XenForo_Application::getOptions()->MasterBadge_memberWidget);

        return $template;
    }

    public static function findAndReplace($content, $replace, $options = array())
    {        
        if (!$options['enabled'])
        {
            return $content;
        }

        switch ($options['position']) 
        {
            case 'visitor_panel':
                $modification = array(
                    'find' => '<xen:if is="{$canViewMemberList}">',
                    'replace' => "{$replace} $0"
                );
                break;
            
            case 'online_users':
                $modification = array(
                    'find' => '<xen:if is="{$profilePosts}">',
                    'replace' => "{$replace} $0"
                );
                break;
                
            case 'profile_post':
                $modification = array(
                    'find' => '<!-- block: forum_stats -->',
                    'replace' => "{$replace} $0"
                );
                break;

            case 'forum_stats':
                $modification = array(
                    'find' => '<xen:include template="sidebar_share_page">',
                    'replace' => "{$replace} $0"
                );
                break;

            case 'share_page':
                $modification = array(
                    'find' => '</xen:hook>',
                    'replace' => "{$replace} $0"
                );
                break; 

            default:
                $modification = array();
                break;
        }

        $modification['action'] = 'str_replace';

        $templateModificationModel = static::_getTemplateModificationModel();

        return $templateModificationModel->applyTemplateModifications($content, array($modification), $status);
    }

    public static function widgetFramework(array $widget, $positionCode, array $params, XenForo_Template_Abstract $template)
    {
        $user = Xenforo_Visitor::getInstance()->toArray();

        $params['progress'] = MasterBadge_Core::getUserLevelProgress($user);
        $contents = $template->create('MasterBadge_widget', $params);

        return $contents->render();
    }

    public static function memberWidgetFramework(array $widget, $positionCode, array $params, XenForo_Template_Abstract $template)
    {
        $userModel = self::_getUserModel();
        $options = XenForo_Application::getOptions();
        
        $conditions = array(
            'user_state' => 'valid',
            'is_banned' => 0
        );

        $fetchOptions = array(
            'limit' => $options->MasterBadge_memberWidgetLimit,
            'order' => 'trophy_points',
            'direction' => 'DESC'
        );

        $params['badgeWidget']['users'] = $userModel->getUsers($conditions, $fetchOptions);
        $contents = $template->create('MasterBadge_member_widget', $params);

        return $contents->render();
    }

    protected static function _getTemplateModificationModel()
    {
        if (!static::$_templateModificationModel)
        {
            static::$_templateModificationModel = XenForo_Model::create('XenForo_Model_TemplateModification');
        }

        return static::$_templateModificationModel;
    }

    protected static function _getUserModel()
    {
        if (!self::$_userModel)
        {
            self::$_userModel = XenForo_Model::create('XenForo_Model_User');
        }

        return self::$_userModel;
    }
}